-----------------------------------------------------------------------------------------------
-- Client Lua Script for HPSStats
-- Copyright (c) Foxykeep. All rights reserved
-----------------------------------------------------------------------------------------------

require "Window"


-----------------------------------------------------------------------------------------------
-- HPSStats Module Definition
-----------------------------------------------------------------------------------------------
local HPSStats = {}

local foxyLib = nil

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local defaultSettings = {
	isVisible = false,
	wndPosition = {50, 30, 210, 215}
}

-----------------------------------------------------------------------------------------------
-- HPSStats Initialization
-----------------------------------------------------------------------------------------------

function HPSStats:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    return o
end

function HPSStats:InitUserSettings()

end

function HPSStats:Init()
	local bHasConfigurateFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"FoxyLib-1.0"
	}
    Apollo.RegisterAddon(self, bHasConfigurateFunction, strConfigureButtonText, tDependencies)
end


-----------------------------------------------------------------------------------------------
-- HPSStats Save & Restore settings
-----------------------------------------------------------------------------------------------

function HPSStats:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end

    local tSave = {}
    tSave.isVisible = self.userSettings.isVisible
    tSave.wndPosition = foxyLib.DeepCopy(self.userSettings.wndPosition)

	return tSave
end

function HPSStats:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end

	self.userSettings.isVisible = tSave.isVisible
	self.userSettings.wndPosition = foxyLib.DeepCopy(tSave.wndPosition)

	-- Data saved in future versions must be lazy restored (if present (~= nil), grab from tSave else
	-- grab from defaultSettings).

	self.onRestoreCalled = true
	self:SetupWndMain()
end


-----------------------------------------------------------------------------------------------
-- HPSStats OnLoad
-----------------------------------------------------------------------------------------------

function HPSStats:OnLoad()
	foxyLib = Apollo.GetPackage("FoxyLib-1.0").tPackage

	self.userSettings = foxyLib.DeepCopy(defaultSettings)

	self.onRestoreCalled = false
	self.onXmlDocLoadedCalled = false

	self.onPrimaryLabelSet = false

	-- Create the form
	self.xmlDoc = XmlDoc.CreateFromFile("HPSStats.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnFrame", self)
end

-----------------------------------------------------------------------------------------------
-- GotHUD OnDocLoaded
-----------------------------------------------------------------------------------------------
function HPSStats:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		-- Load the settings window
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "HPSStatsWindow", nil, self)
		if not self.wndMain then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.wndMain:Show(false)

		-- Register handlers for events, slash commands and timer, etc.
		Apollo.RegisterSlashCommand("HPSStats", "ToggleMainUI", self)
		Apollo.RegisterSlashCommand("hpsstats", "ToggleMainUI", self)

		self.onXmlDocLoadedCalled = true
		self:SetupWndMain()
	end
end


-----------------------------------------------------------------------------------------------
-- Wnd position
-----------------------------------------------------------------------------------------------

function HPSStats:SetupWndMain()
	if not self.onRestoreCalled or not self.onXmlDocLoadedCalled then
		return
	end

	local anchors = self.userSettings.wndPosition
	self.wndMain:SetAnchorOffsets(anchors[1], anchors[2], anchors[3], anchors[4])
end

function HPSStats:summation(intStart, intEnd, step)
	local sum = 0
	while intStart <= intEnd do
		sum = sum + math.pow(0.9997211, intStart)
		intStart = intStart + step
	end
	return sum
end

-----------------------------------------------------------------------------------------------
-- Main loop (on every 4 frames)
-----------------------------------------------------------------------------------------------

function HPSStats:OnFrame()
	if self.userSettings.isVisible then
		-- We are in not visible. Nothing to do here. Let's just hide if we are currently visible
		if self.wndMain:IsVisible() then
			self.wndMain:Show(false)
		end
		return
	end

	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		-- We don't have the player object yet.
		return
	end

	-- Show the window if not currently visible
	if not self.wndMain:IsVisible() then
		self.wndMain:Show(true)
	end

	-- Set the primary stats label if not done yet
	if not self.onPrimaryLabelSet then
		self.wndMain:FindChild("LabelPrimary"):SetText("Insight") --Apollo.GetString("CRB_Insight")
	end

	-- Save the position
	local left, top, right, bottom = self.wndMain:GetAnchorOffsets()
	self.userSettings.wndPosition  = { left, top, right, bottom }
	if unitPlayer:GetSupportPower() >= 3600 then
	    -- AP ROT, Effektiv Orange
		self.wndMain:FindChild("ValueSP"):SetTextColor("ChannelDebug")
		self.wndMain:FindChild("ValueEffectiveSP"):SetTextColor("ItemQuality_Legendary")
		self.wndMain:FindChild("ValueEffectiveSP"):SetText(math.floor(HPSStats:summation(0, unitPlayer:GetSupportPower()-3600, 1) + 3600))
	else
		-- AP grn
		self.wndMain:FindChild("ValueSP"):SetTextColor("ItemQuality_Good")
		self.wndMain:FindChild("ValueEffectiveSP"):SetTextColor("ItemQuality_Good")
		self.wndMain:FindChild("ValueEffectiveSP"):SetText(math.floor(unitPlayer:GetSupportPower()))
	end
	self.wndMain:FindChild("ValueSP"):SetText(math.floor(unitPlayer:GetSupportPower()))
	local primaryValue = math.floor(unitPlayer:GetUnitProperty(4).fValue)
	self.wndMain:FindChild("ValuePrimary"):SetText(primaryValue)
	local critHitValue = math.floor((unitPlayer:GetCritChance() + 0.000005) * 10000) / 100
	self.wndMain:FindChild("ValueCritHit"):SetText(critHitValue .. "%")
	local critSevValue = math.floor((unitPlayer:GetCritSeverity() + 0.000005) * 10000) / 100
	self.wndMain:FindChild("ValueCritSev"):SetText(critSevValue .. "%")
	local focusRecovery = String_GetWeaselString(Apollo.GetString("Character_PerSecLabel"), Apollo.FormatNumber(unitPlayer:GetManaRegenInCombat() * 2, 2, true))
	self.wndMain:FindChild("ValueFocusRec"):SetText(focusRecovery)
	local focusCostRed = String_GetWeaselString(Apollo.GetString("Character_PercentAppendLabel"), Apollo.FormatNumber(math.floor((math.abs(unitPlayer:GetManaCostModifier() -1) + 0.000005) * 10000) / 100, 2, true))
	self.wndMain:FindChild("ValueFocusCostRed"):SetText(focusCostRed)
end


-----------------------------------------------------------------------------------------------
-- Toggle visibility
-----------------------------------------------------------------------------------------------

function HPSStats:ToggleMainUI()
	self.userSettings.isVisible = not self.userSettings.isVisible
end

-----------------------------------------------------------------------------------------------
-- HPSStats Instance
-----------------------------------------------------------------------------------------------
local HPSStatsInst = HPSStats:new()
HPSStatsInst:Init()